@@ -2,7 +2,7 @@ package FusionInventory::Agent::Tools;
 
 use strict;
 use warnings;
-use base 'Exporter';
+use parent 'Exporter';
 
 use Encode qw(encode);
 use English qw(-no_match_vars);
@@ -13,7 +13,16 @@ use File::Which;
 use Memoize;
 use UNIVERSAL::require;
 
+use FusionInventory::Agent::Tools::Expiration;
+
+# Keep a copy of @ARGV, only for Provider inventory
+BEGIN {
+    our $ARGV = [ @ARGV ];
+}
+our $ARGV;
+
 our @EXPORT = qw(
+    first
     getDirectoryHandle
     getFileHandle
     getFormatedLocalTime
@@ -21,8 +30,10 @@ our @EXPORT = qw(
     getFormatedDate
     getCanonicalManufacturer
     getCanonicalSpeed
+    getCanonicalInterfaceSpeed
     getCanonicalSize
     getSanitizedString
+    getUtf8String
     trimWhitespace
     getFirstLine
     getFirstMatch
@@ -45,14 +56,24 @@ our @EXPORT = qw(
     slurp
 );
 
-my $nowhere = $OSNAME eq 'MSWin32' ? 'nul' : '/dev/null';
-
-# this trigger some errors on Perl 5.12/Win32:
+# this trigger some errors under win32:
 # Anonymous function called in forbidden scalar context
 if ($OSNAME ne 'MSWin32') {
     memoize('canRun');
 }
 
+# Avoid List::Util dependency re-using 'any' sub as template
+sub first (&@) { ## no critic (SubroutinePrototypes)
+    my $f = shift;
+
+    ## no critic (ExplicitReturnUndef)
+
+    foreach ( @_ ) {
+        return $_ if $f->();
+    }
+    return undef;
+}
+
 sub getFormatedLocalTime {
     my ($time) = @_;
 
@@ -96,24 +117,25 @@ sub getCanonicalManufacturer {
         TMx86        => 'Transmeta',
         TransmetaCPU => 'Transmeta',
         CyrixInstead => 'Cyrix',
-        CentaurHauls => 'VIA'
+        CentaurHauls => 'VIA',
+        HygonGenuine => 'Hygon'
     );
 
     if (exists $manufacturers{$manufacturer}) {
         $manufacturer = $manufacturers{$manufacturer};
     } elsif ($manufacturer =~ /(
-        maxtor    |
-        sony      |
-        compaq    |
-        ibm       |
-        toshiba   |
-        fujitsu   |
-        lg        |
-        samsung   |
-        nec       |
-        transcend |
-        matshita  |
-        hitachi   |
+        maxtor     |
+        sony       |
+        compaq     |
+        ibm        |
+        toshiba    |
+        fujitsu    |
+        \blg\b     |
+        samsung    |
+        nec        |
+        transcend  |
+        matshita   |
+        hitachi    |
         pioneer
     )/xi) {
         $manufacturer = ucfirst(lc($1));
@@ -125,6 +147,10 @@ sub getCanonicalManufacturer {
         $manufacturer = "Seagate";
     } elsif ($manufacturer =~ /^(HD|IC|HU|HGST)/) {
         $manufacturer = "Hitachi";
+    } elsif ($manufacturer =~ /^APPLE/i) {
+        $manufacturer = "Apple";
+    } elsif ($manufacturer =~ /^OPTIARC/i) {
+        $manufacturer = "Sony";
     }
 
     return $manufacturer;
@@ -137,16 +163,40 @@ sub getCanonicalSpeed {
 
     return undef unless $speed;
 
+    return $speed if $speed =~ /^([,.\d]+)$/;
+
     return 400 if $speed =~ /^PC3200U/;
 
-    return undef unless $speed =~ /^([,.\d]+) \s? (\S+)$/x;
+    return undef unless $speed =~ /^([,.\d]+) \s? (\S+)/x;
+    my $value = $1;
+    my $unit = lc($2);
+
+    # Remark: we need to return speed in MHz. Even if MT/s is not accurately
+    # equivalent to MHz, we nned to return the value so server can extract it
+    return
+        $unit eq 'ghz'  ? $value * 1000 :
+        $unit eq 'mhz'  ? $value        :
+        $unit eq 'mt/s' ? $value        :
+                          undef         ;
+}
+
+sub getCanonicalInterfaceSpeed {
+    # Expected unit is Mb/s as specified in inventory protocol
+    my ($speed) = @_;
+
+    ## no critic (ExplicitReturnUndef)
+
+    return undef unless $speed;
+
+    return undef unless $speed =~ /^([,.\d]+) \s? (\S\S)\S*$/x;
     my $value = $1;
     my $unit = lc($2);
 
     return
-        $unit eq 'ghz' ? $value * 1000 :
-        $unit eq 'mhz' ? $value        :
-                         undef         ;
+        $unit eq 'gb' ? $value * 1000         :
+        $unit eq 'mb' ? $value                :
+        $unit eq 'kb' ? int($value / 1000)    :
+                        undef                 ;
 }
 
 sub getCanonicalSize {
@@ -164,6 +214,7 @@ sub getCanonicalSize {
     return undef unless $size =~ /^([,.\d]+) (\S+)$/x;
     my $value = $1;
     my $unit = lc($2);
+    $value =~ s/,/\./;
 
     return
         $unit eq 'tb'    ? $value * $base * $base        :
@@ -192,14 +243,11 @@ sub compareVersion {
         );
 }
 
-sub getSanitizedString {
+sub getUtf8String {
     my ($string) = @_;
 
     return unless defined $string;
 
-    # clean control caracters
-    $string =~ s/[[:cntrl:]]//g;
-
     # encode to utf-8 if needed
     if (!Encode::is_utf8($string) && $string !~ m/\A(
           [\x09\x0A\x0D\x20-\x7E]           # ASCII
@@ -217,6 +265,17 @@ sub getSanitizedString {
     return $string;
 }
 
+sub getSanitizedString {
+    my ($string) = @_;
+
+    return unless defined $string;
+
+    # clean control caracters
+    $string =~ s/[[:cntrl:]]//g;
+
+    return getUtf8String($string);
+}
+
 sub trimWhitespace {
     my ($value) = @_;
     $value =~ s/^\s+//;
@@ -241,6 +300,7 @@ sub getDirectoryHandle {
     return $handle;
 }
 
+my $cmdtemplate = $OSNAME eq 'MSWin32' ? "%s 2>nul" : "exec %s 2>/dev/null";
 sub getFileHandle {
     my (%params) = @_;
 
@@ -265,12 +325,16 @@ sub getFileHandle {
             local $ENV{LANG} = 'C';
             # Ignore 'Broken Pipe' warnings on Solaris
             local $SIG{PIPE} = 'IGNORE' if $OSNAME eq 'solaris';
-            if (!open $handle, '-|', $params{command} . " 2>$nowhere") {
+            my $command = sprintf($cmdtemplate, $params{command});
+            my $cmdpid  = open($handle, '-|', $command);
+            if (!$cmdpid) {
                 $params{logger}->error(
                     "Can't run command $params{command}: $ERRNO"
                 ) if $params{logger};
                 return;
             }
+            # Kill command if a timeout was set
+            $SIG{ALRM} = sub { kill 'KILL', $cmdpid ; die "alarm\n"; } if $SIG{ALRM};
             last SWITCH;
         }
         if ($params{string}) {
@@ -292,7 +356,7 @@ sub getFirstLine {
     my $result = <$handle>;
     close $handle;
 
-    chomp $result if $result;
+    chomp $result if defined $result;
     return $result;
 }
 
@@ -308,7 +372,7 @@ sub getLastLine {
     }
     close $handle;
 
-    chomp $result if $result;
+    chomp $result if defined $result;
     return $result;
 }
 
@@ -326,7 +390,7 @@ sub getFirstMatch {
     }
     close $handle;
 
-    return wantarray ? @results : $results[0];
+    return wantarray ? @results : first { defined $_ } @results;
 }
 
 sub getAllLines {
@@ -374,7 +438,7 @@ sub hex2char {
     my ($value) = @_;
 
     ## no critic (ExplicitReturnUndef)
-    return undef unless $value;
+    return undef unless defined $value;
     return $value unless $value =~ /^0x/;
 
     $value =~ s/^0x//; # drop hex prefix
@@ -385,7 +449,7 @@ sub hex2dec {
     my ($value) = @_;
 
     ## no critic (ExplicitReturnUndef)
-    return undef unless $value;
+    return undef unless defined $value;
     return $value unless $value =~ /^0x/;
 
     return oct($value);
@@ -395,7 +459,7 @@ sub dec2hex {
     my ($value) = @_;
 
     ## no critic (ExplicitReturnUndef)
-    return undef unless $value;
+    return undef unless defined $value;
     return $value if $value =~ /^0x/;
 
     return sprintf("0x%x", $value);
@@ -463,8 +527,12 @@ sub runFunction {
     my $result;
     eval {
         local $SIG{ALRM} = sub { die "alarm\n" };
+
         # set a timeout if needed
-        alarm $params{timeout} if $params{timeout};
+        if ($params{timeout}) {
+            alarm $params{timeout};
+            setExpirationTime(timeout => $params{timeout});
+        }
 
         no strict 'refs'; ## no critic (ProhibitNoStrict)
         $result = &{$params{module} . '::' . $params{function}}(
@@ -472,7 +540,10 @@ sub runFunction {
             ref $params{params} eq 'ARRAY' ? @{$params{params}} :
                                                $params{params}
         );
+
+        # Reset timeout
         alarm 0;
+        setExpirationTime();
     };
 
     if ($EVAL_ERROR) {
@@ -540,6 +611,10 @@ Returns a normalized manufacturer value for given one.
 
 Returns a normalized speed value (in Mhz) for given one.
 
+=head2 getCanonicalInterfaceSpeed($speed)
+
+Returns a normalized network interface speed value (in Mb/s) for given one.
+
 =head2 getCanonicalSize($size, $base)
 
 Returns a normalized size value (in Mb) for given one.
@@ -549,6 +624,10 @@ Returns a normalized size value (in Mb) for given one.
 Returns the input stripped from any control character, properly encoded in
 UTF-8.
 
+=head2 getUtf8String($string)
+
+Returns the input properly encoded in UTF-8.
+
 =head2 trimWhitespace($string)
 
 Remove leading and trailing whitespace, and fold multiple whitespace
@@ -696,6 +775,11 @@ hexadecimal prefix, the unconverted value otherwise. Eg. 65 -> 0x41, 0x41 ->
 Returns a true value if any item in LIST meets the criterion given through
 BLOCK.
 
+=head2 first BLOCK LIST
+
+Returns the first value from LIST meeting the criterion given through BLOCK or
+undef.
+
 =head2 all BLOCK LIST
 
 Returns a true value if all items in LIST meet the criterion given through
@@ -743,3 +827,7 @@ on the Operating System.
 =head2 slurp($file)
 
 Return the content of a given file.
+
+=head2 isParamArrayAndFilled($hash, $paramName)
+
+Return if $hash has a key $paramName which value is a ARRAY ref not empty.
